/**
 * Copyright (C) cedarsoft GmbH.
 *
 * Licensed under the GNU General Public License version 3 (the "License")
 * with Classpath Exception; you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *         http://www.cedarsoft.org/gpl3ce
 *         (GPL 3 with Classpath Exception)
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3 only, as
 * published by the Free Software Foundation. cedarsoft GmbH designates this
 * particular file as subject to the "Classpath" exception as provided
 * by cedarsoft GmbH in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 3 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 3 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact cedarsoft GmbH, 72810 Gomaringen, Germany,
 * or visit www.cedarsoft.com if you need additional information or
 * have any questions.
 */

package com.cedarsoft.gdao;

import javax.annotation.Nonnull;
import org.springframework.transaction.support.AbstractPlatformTransactionManager;
import org.springframework.transaction.support.TransactionTemplate;

import java.lang.Override;
import java.util.List;

/**
 * Abstract base class for services.
 */
public abstract class AbstractService<T> extends AbstractGenericDao<T> {
  @Nonnull
  protected final TransactionTemplate transactionTemplate;
  @Nonnull
  protected final GenericDao<T> dao;

  /**
   * Creates a new service
   *
   * @param dao                the dao
   * @param transactionManager the transaction manager that is used
   */
  protected AbstractService( @Nonnull GenericDao<T> dao, @Nonnull AbstractPlatformTransactionManager transactionManager ) {
    this.dao = dao;
    this.transactionTemplate = new TransactionTemplate( transactionManager );
  }

  @Nonnull
  protected TransactionTemplate getTransactionTemplate() {
    return transactionTemplate;
  }

  /**
   * Returns the underlying dao
   *
   * @return the dao
   */
  @Nonnull
  public GenericDao<T> getDao() {
    return dao;
  }

  @Override
  @Nonnull
  public List<? extends T> findAll() {
    return dao.findAll();
  }

  @Override
  public int getCount() {
    return dao.getCount();
  }

  @Override
  @Nonnull
  public <LT extends T> Long save( @Nonnull LT newInstance ) {
    return dao.save( newInstance );
  }

  @Override
  @Nonnull
  public T findById( @Nonnull Long id ) {
    return dao.findById( id );
  }

  @Override
  public <LT extends T> void update( @Nonnull LT transientObject ) {
    dao.update( transientObject );
  }

  @Override
  public <LT extends T> void saveOrUpdate( @Nonnull LT transientObject ) {
    dao.saveOrUpdate( transientObject );
  }

  @Override
  public <LT extends T> void delete( @Nonnull LT persistentObject ) {
    dao.delete( persistentObject );
  }

  @Override
  @Nonnull
  public <T> T find( @Nonnull Finder<T> finder ) {
    return dao.find( finder );
  }

  @Override
  public void shutdown() {
    dao.shutdown();
  }
}
