/**
 * Copyright (C) cedarsoft GmbH.
 *
 * Licensed under the GNU General Public License version 3 (the "License")
 * with Classpath Exception; you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *         http://www.cedarsoft.org/gpl3ce
 *         (GPL 3 with Classpath Exception)
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3 only, as
 * published by the Free Software Foundation. cedarsoft GmbH designates this
 * particular file as subject to the "Classpath" exception as provided
 * by cedarsoft GmbH in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 3 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 3 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact cedarsoft GmbH, 72810 Gomaringen, Germany,
 * or visit www.cedarsoft.com if you need additional information or
 * have any questions.
 */

package com.cedarsoft.gdao;

import com.cedarsoft.cache.Cache;
import com.cedarsoft.cache.HashedCache;
import javax.annotation.Nonnull;
import org.springframework.transaction.support.AbstractPlatformTransactionManager;

import java.lang.Override;

/**
 * Manages the services
 */
public class DefaultGenericServiceManager implements GenericServiceManager {
  @Nonnull
  private final GenericDaoManager daoManager;
  @Nonnull
  private final AbstractPlatformTransactionManager transactionManager;

  @SuppressWarnings( {"MismatchedQueryAndUpdateOfCollection"} )
  @Nonnull
  private final Cache<Class<Object>, GenericService<Object>> serviceCache;

  /**
   * Should only be necessary for testing purposes
   *
   * @param transactionManager the transaction manager
   * @param daoManager         the dao manager
   * @param factory            the factory
   */
  protected DefaultGenericServiceManager( @Nonnull AbstractPlatformTransactionManager transactionManager, @Nonnull GenericDaoManager daoManager, @Nonnull Cache.Factory<Class<Object>, GenericService<Object>> factory ) {
    serviceCache = new HashedCache<Class<Object>, GenericService<Object>>( factory );
    this.transactionManager = transactionManager;
    this.daoManager = daoManager;

  }

  /**
   * Creates a new service manager
   *
   * @param transactionManager the transaction manager
   * @param daoManager         the dao manager
   */
  public DefaultGenericServiceManager( @Nonnull AbstractPlatformTransactionManager transactionManager, @Nonnull GenericDaoManager daoManager ) {
    this.transactionManager = transactionManager;
    this.daoManager = daoManager;
    serviceCache = new HashedCache<Class<Object>, GenericService<Object>>( new Cache.Factory<Class<Object>, GenericService<Object>>() {
      @Override
      @Nonnull
      public GenericService<Object> create( @Nonnull Class<Object> key ) {
        return new GenericServiceImpl<Object>( DefaultGenericServiceManager.this.daoManager.getDao( key ), DefaultGenericServiceManager.this.transactionManager );
      }
    } );
  }

  /**
   * Returns the service for the given type
   *
   * @param type the type
   * @return the service for the given type
   */
  @Override
  @Nonnull
  public <T> GenericService<T> getService( @Nonnull Class<T> type ) {
    return ( GenericService<T> ) serviceCache.get( type );
  }
}
