/**
 * Copyright (C) cedarsoft GmbH.
 *
 * Licensed under the GNU General Public License version 3 (the "License")
 * with Classpath Exception; you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *         http://www.cedarsoft.org/gpl3ce
 *         (GPL 3 with Classpath Exception)
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3 only, as
 * published by the Free Software Foundation. cedarsoft GmbH designates this
 * particular file as subject to the "Classpath" exception as provided
 * by cedarsoft GmbH in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 3 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 3 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact cedarsoft GmbH, 72810 Gomaringen, Germany,
 * or visit www.cedarsoft.com if you need additional information or
 * have any questions.
 */

package com.cedarsoft.gdao.async;

import com.cedarsoft.gdao.GenericDao;
import com.cedarsoft.gdao.GenericService;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;

import java.lang.Override;

/**
 *
 */
public class AsynchronousService<T> extends AsynchronousDao<T> implements GenericService<T> {
  @Override
  @Nullable
  public <R> R perform( @Nonnull final ServiceCallback<T, R> callback ) {
    return asyncCallSupport.<R>invokeNullable( new DaoAction<T, R>() {
      @Override
      public R execute( @Nonnull GenericDao<T> dao ) {
        GenericService<T> service = ( GenericService<T> ) dao;
        return callback.perform( service );
      }
    } );
  }

  public void initializeDelegatingService( @Nonnull GenericService<T> delegatingService ) {
    initializeDelegatingDao( delegatingService );
  }

  @Override
  public void initializeDelegatingDao( @Nonnull GenericDao<T> delegatingDao ) {
    if ( !( delegatingDao instanceof GenericService ) ) {
      throw new IllegalArgumentException( "AsynchronousService must be initialized with an GenericService as delegating dao" );
    }
    super.initializeDelegatingDao( delegatingDao );
  }
}
